@@ -110,7 +110,7 @@ master_clock_schedule_forced_stages_updates (ClutterMasterClockGdk *master_clock
   g_hash_table_iter_init (&iter, master_clock->stage_to_clock);
   while (g_hash_table_iter_next (&iter, &stage, &frame_clock))
     gdk_frame_clock_request_phase (GDK_FRAME_CLOCK (frame_clock),
-                                   GDK_FRAME_CLOCK_PHASE_UPDATE);
+                                   GDK_FRAME_CLOCK_PHASE_PAINT);
 }
 
 static void
@@ -131,7 +131,7 @@ master_clock_schedule_stage_update (ClutterMasterClockGdk *master_clock,
    * we have no choice, we need to advance the timelines for the next
    * frame. */
   if (master_clock->timelines != NULL)
-    gdk_frame_clock_request_phase (frame_clock, GDK_FRAME_CLOCK_PHASE_UPDATE);
+    gdk_frame_clock_request_phase (frame_clock, GDK_FRAME_CLOCK_PHASE_PAINT);
 }
 
 static void
@@ -310,6 +310,7 @@ clutter_master_clock_gdk_remove_stage_clock (ClutterMasterClockGdk *master_clock
                                                 clutter_master_clock_gdk_update,
                                                 master_clock);
           g_hash_table_remove (master_clock->clock_to_stage, frame_clock);
+          g_list_free (stages);
         }
       else
         {
@@ -340,7 +341,7 @@ clutter_master_clock_gdk_add_stage_clock (ClutterMasterClockGdk *master_clock,
       g_hash_table_insert (master_clock->clock_to_stage, g_object_ref (frame_clock),
                            g_list_append (NULL, stage));
 
-      g_signal_connect (frame_clock, "update",
+      g_signal_connect (frame_clock, "paint",
                         G_CALLBACK (clutter_master_clock_gdk_update),
                         master_clock);
     }
@@ -348,7 +349,7 @@ clutter_master_clock_gdk_add_stage_clock (ClutterMasterClockGdk *master_clock,
     stages = g_list_append (stages, stage);
 
   if (master_clock->timelines != NULL)
-    _clutter_master_clock_start_running ((ClutterMasterClock *) clock);
+    _clutter_master_clock_start_running ((ClutterMasterClock *) master_clock);
 }
 
 static void
@@ -384,11 +385,12 @@ clutter_master_clock_gdk_listen_to_stage (ClutterMasterClockGdk *master_clock,
 }
 
 static void
-clutter_master_clock_gdk_stage_realized (ClutterStage          *stage,
-                                         GParamSpec            *spec,
-                                         ClutterMasterClockGdk *master_clock)
+clutter_master_clock_gdk_stage_visibility (ClutterStage          *stage,
+                                           GParamSpec            *spec,
+                                           ClutterMasterClockGdk *master_clock)
 {
-  if (clutter_actor_is_realized (CLUTTER_ACTOR (stage)))
+  ClutterActor *actor = CLUTTER_ACTOR (stage);
+  if (clutter_actor_is_mapped (actor))
     clutter_master_clock_gdk_listen_to_stage (master_clock, stage);
   else
     clutter_master_clock_gdk_remove_stage_clock (master_clock, stage);
@@ -399,11 +401,11 @@ clutter_master_clock_gdk_stage_added (ClutterStageManager   *manager,
                                       ClutterStage          *stage,
                                       ClutterMasterClockGdk *master_clock)
 {
-  g_signal_connect (stage, "notify::realized",
-                    G_CALLBACK (clutter_master_clock_gdk_stage_realized),
+  g_signal_connect (stage, "notify::mapped",
+                    G_CALLBACK (clutter_master_clock_gdk_stage_visibility),
                     master_clock);
 
-  clutter_master_clock_gdk_listen_to_stage (master_clock, stage);
+  clutter_master_clock_gdk_stage_visibility (stage, NULL, master_clock);
 }
 
 static void
@@ -414,7 +416,7 @@ clutter_master_clock_gdk_stage_removed (ClutterStageManager   *manager,
   clutter_master_clock_gdk_remove_stage_clock (master_clock, stage);
 
   g_signal_handlers_disconnect_by_func (stage,
-                                        clutter_master_clock_gdk_stage_realized,
+                                        clutter_master_clock_gdk_stage_visibility,
                                         master_clock);
 }
 
@@ -461,7 +463,7 @@ clutter_master_clock_gdk_init (ClutterMasterClockGdk *self)
   const GSList *stages, *l;
 
   self->clock_to_stage = g_hash_table_new_full (g_direct_hash, g_direct_equal,
-                                                g_object_unref, (GDestroyNotify) g_list_free);
+                                                g_object_unref, NULL);
   self->stage_to_clock = g_hash_table_new_full (g_direct_hash, g_direct_equal,
                                                 NULL, g_object_unref);
 
@@ -475,7 +477,6 @@ clutter_master_clock_gdk_init (ClutterMasterClockGdk *self)
   for (l = stages; l; l = l->next)
     clutter_master_clock_gdk_stage_added (manager, l->data, self);
 
-
   if (G_UNLIKELY (clutter_paint_debug_flags & CLUTTER_DEBUG_CONTINUOUS_REDRAW))
     g_warning ("Continuous redraw is not supported with the GDK backend.");
 }
